from .rule import Rule

def parse_document_rules(config):
    document_names = parse_documents_name(config)
    default_rules = parse_default_document_rules(config)
    document_rules = {}
    for document_name in document_names:
        document_rules[document_name] = parse_specific_document_rules(config[document_name], default_rules)
    return document_rules


def parse_documents_name(config):
    document_names = []
    for key, value in config.items():
        if not key in Rule.reserved:
            document_names.append(key)
    return document_names


def parse_default_document_rules(config):
    default_rule = Rule(config)
    return default_rule


def parse_specific_document_rules(config, default_rules):
    specific = default_rules.copy()
    specific.set_config(config)
    return specific



import pretty_output, argparse, traceback

from karamel.command import Command

from kooki.jars import search_file, search_recipe, search_jar
from kooki.config import read_config_file
from kooki.rule_parser import parse_document_rules
from kooki.extension import load_extensions, caller
from kooki.exception import KookiException

from collections import OrderedDict

__command__ = 'bake'
__description__ = 'Bake a kooki'

output = []

def output_search_start():
    global output
    output = []


def output_search(path, fullpath):
    if fullpath:
        output.append({'name': path, 'status': '[found]', 'path': fullpath})
    else:
        output.append({'name': path, 'status': ('[missing]', 'red'), 'path': ''})


def output_search_finish():
    pretty_output.infos(output, [('name', 'blue'), ('status', 'green'), ('path', 'cyan')])


class BakeCommand(Command):

    def __init__(self):
        super(BakeCommand, self).__init__(__command__, __description__)
        self.add_argument('documents', nargs='*')
        self.add_argument('--config-file', default='kooki.yaml')
        self.add_argument('-h', '--help', action='help', default=argparse.SUPPRESS, help='Show this help message and exit.')
        self.add_argument('-d', '--debug', help='Show information to help debug the bake processing', action='store_true')
        self.add_argument('--no-color', help='The output has no color.', action='store_true')
        self.add_argument('--no-output', help='There is no output.', action='store_true')

    def callback(self, args):
        try:
            pretty_output.set_output_policy(not args.no_output)
            pretty_output.set_color_policy(not args.no_color)
            pretty_output.set_debug_policy(args.debug)
            pretty_output.command('bake')

            self.real_call(args)

        except KookiException as e:
            pretty_output.error_step('Errors')
            pretty_output.error(e)

        except Exception as e:
            pretty_output.error_step('Errors')
            pretty_output.error(traceback.format_exc()[:-1])

        finally:
            pretty_output.command()

    def real_call(self, args):
        config = read_config_file(args.config_file)
        document_rules = parse_document_rules(config)

        if args.documents == []:
            documents = document_rules
        else:
            documents = {}
            for document_name in args.documents:
                if document_name in document_rules:
                    documents[document_name] = document_rules[document_name]
                else:
                    raise Exception('Bad document')

        for name, document in documents.items():
            recipe = document.recipe
            jars = document.jars

            pretty_output.start_document(name)

            # jars
            pretty_output.start_step('jars')
            output_search_start()
            full_path_jars = []
            for jar in document.jars:
                jar_full_path = search_jar(jar)
                output_search(jar, jar_full_path)
                full_path_jars.append(jar_full_path)
            document.jars = full_path_jars
            output_search_finish()

            # extensions
            pretty_output.start_step('extensions')
            output_search_start()
            extensions = load_extensions(document.jars, document.recipe)
            new_extensions = {}
            for extension_name, extension_path in extensions.items():
                new_extensions[extension_name] = caller(extension_path)
                output_search(extension_name, extension_path)
            output_search_finish()

            # template
            pretty_output.start_step('template')
            output_search_start()
            file_full_path = search_file(jars, recipe, document.template)
            output_search(document.template, file_full_path)
            if file_full_path:
                with open(file_full_path,'r') as f:
                    file_read = f.read()
                document.template = file_read
            output_search_finish()

            # metadata
            pretty_output.start_step('metadata')
            metadata_full_path = {}
            output_search_start()
            for metadata in document.metadata:
                file_full_path = search_file(jars, recipe, metadata)
                with open(file_full_path,'r') as f:
                    file_read = f.read()
                metadata_full_path[file_full_path] = file_read
                output_search(metadata, file_full_path)
            document.metadata = metadata_full_path
            output_search_finish()

            # content
            pretty_output.start_step('content')
            content_full_path = OrderedDict()
            output_search_start()
            for content in document.content:
                file_full_path = search_file(jars, recipe, content)
                with open(file_full_path,'r') as f:
                    file_read = f.read()
                content_full_path[file_full_path] = file_read
                output_search(content, file_full_path)
            document.content = content_full_path
            output_search_finish()

            # recipe
            pretty_output.start_step('recipe')
            output_search_start()
            file_full_path = search_recipe(recipe)
            output_search(document.recipe, file_full_path)
            output_search_finish()

            if file_full_path:
                with open(file_full_path,'r') as f:
                    recipe_read = f.read()
                    variables = {}
                    exec(recipe_read, variables)
                    if 'recipe' in variables:
                        recipe = variables['recipe']
                    recipe(document, new_extensions)
            else:
                raise(Exception('bad recipe'))



from karamel.command import Command

__command__ = 'update'
__description__ = 'Update a kooki project'


class UpdateCommand(Command):

    def __init__(self):
        super(UpdateCommand, self).__init__(__command__, __description__)
        self.add_argument('-f', '--config-file', default='kooki.yaml')

    def callback(self, args):
        pass




class Rule:

    reserved = ['name', 'template', 'recipe', 'jars', 'metadata', 'content']

    def __init__(self, config={}):
        self.name = ''
        self.template = ''
        self.recipe = ''
        self.jars = []
        self.metadata = []
        self.content = []
        self.set_config(config)

    def set_config(self, config):
        for key, value in config.items():
            if key == 'name':
                self.name = value
            elif key == 'template':
                self.template = value
            elif key == 'recipe':
                self.recipe = value
            elif key == 'jars':
                self.jars = value
            elif key == 'metadata':
                self.metadata = value
            elif key == 'content':
                self.content = value

    def copy(self):
        rule = Rule()
        rule.name = self.name
        rule.template = self.template
        rule.recipe = self.recipe
        rule.jars = self.jars
        rule.metadata = self.metadata
        rule.content = self.content
        return rule



from collections import OrderedDict

from .metadata import *
from .template import *


def apply_custom(parts, custom):
    result = []
    for index, part in enumerate(parts):
        result.append(custom(part, index))
    return result


def apply_merge(data):
    result = None

    if isinstance(data, list):
        result = ''
        for content in data:
            result += content

    elif isinstance(data, OrderedDict):
        result = ''
        for key, content in OrderedDict(data).items():
            result += content

    else:
        raise Exception('merging bad data type {}'.format(type(data)))

    return result


def apply_split(text, separator):
    parts = text.split(separator)
    return parts



import os, em
from kooki.tools import get_front_matter

def load_extensions(jars, recipe):
    current = os.getcwd()
    extensions = {}
    path = os.path.join(current, 'extensions')
    if os.path.isdir(path):
        load_extensions_in_directory(path, extensions)
    for jar in jars:
        path = os.path.join(jar, recipe, 'extensions')
        if os.path.isdir(path):
            load_extensions_in_directory(path, extensions)
        path = os.path.join(jar, 'extensions')
        if os.path.isdir(path):
            load_extensions_in_directory(path, extensions)
    return extensions


def load_extensions_in_directory(directory, extensions):
    for file_name in os.listdir(directory):
        path = os.path.join(directory, file_name)
        extension_name = os.path.splitext(file_name)[0]
        if not extension_name in extensions:
            extensions[extension_name] = path


def caller(file_full_path, metadata={}):
    def call(*args, **kwargs):
        with open(file_full_path,'r') as f:
            file_content = f.read()
            interpreter = em.Interpreter()
            interpreter.setPrefix('@')
            front_matter, new_content = get_front_matter(file_content)
            metadata_copy = metadata.copy()
            metadata_copy.update(front_matter)
            metadata_copy.update({**kwargs})
            new_metadata = metadata_copy
            try:
                ret_content = interpreter.expand(new_content, new_metadata)
            except:
                ret_content = new_content
            return ret_content
    return call



from termcolor import colored
import logging

logger = logging.getLogger()
logger.setLevel(logging.INFO)
stream_handler = logging.StreamHandler()
formatter = logging.Formatter('%(message)s')
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.INFO)
logger.addHandler(stream_handler)

_color_policy = True
_output_policy = True
_debug_policy = False
_message = ''


def output_control(func):
    def wrapper(*args, **kwargs):
        if _output_policy:
            func(*args, **kwargs)
    return wrapper


def set_color_policy(color_policy):
    global _color_policy
    _color_policy = color_policy


def set_output_policy(output_policy):
    global _output_policy
    _output_policy = output_policy


def set_debug_policy(debug):
    global _debug_policy
    _debug_policy = debug


@output_control
def color(full_text):
    results = full_text.split('[')
    for result in results:
        if result and result != '':
            second_results = result.split(']')
            color, text = second_results[0], second_results[1]
            _print_colored(text, color, end='')


@output_control
def title(name):
    _step(name, 'white', '-')


@output_control
def command(name=None):
    if name:
        _step(name, 'white', '=', '[', ']')
    else:
        _step('', 'white', '=', no_space=True)


@output_control
def start_document(name):
    _step(name, 'white', '-', '', '')


@output_control
def start_step(name):
    _step(name, 'yellow', '-')


@output_control
def error_step(name):
    _step(name, 'red', '-')


@output_control
def category(message):
    _print_colored('[' + message + ']', 'magenta')


@output_control
def error(message):
    _print_colored(message, 'red')


@output_control
def warning(message):
    _print_colored(message, 'orange')


@output_control
def info(message):
    _print_colored(message, 'cyan')


def _step(name, text_color, character, before='', after='', no_space=False, fixed_width=80):

    str_size = len(name)

    if (str_size % 2) == 0:
        number_of_dash_left = int((fixed_width - str_size) / 2)
        number_of_dash_right = number_of_dash_left

    else:
        number_of_dash_left = int((fixed_width - str_size) / 2)
        number_of_dash_right = number_of_dash_left + 1

    number_of_dash_left -= len(before)
    number_of_dash_right -= len(after)

    space = ' '
    if no_space:
        space = ''
        number_of_dash_left += 1
        number_of_dash_right += 1

    dash_left = ''
    for i in range(0, number_of_dash_left):
        dash_left += character
    dash_left += before

    dash_right = after
    for i in range(0, number_of_dash_right):
        dash_right += character

    _print_colored('{0}{3}{1}{3}{2}'.format(dash_left, name, dash_right, space), text_color)


@output_control
def push(message):
    pass


@output_control
def infos(infos, attrs):
    text = ''
    spaces = {}
    shift = 2

    for info in infos:
        for attr in attrs:
            name = attr[0]
            if name in spaces:
                if len(info[name]) + shift > spaces[name]:
                    spaces[name] = len(info[name]) + shift
            else:
                spaces[name] = len(info[name]) + shift

    for info in infos:
        for attr in attrs:
            name = attr[0]
            color = attr[1]
            text = ''
            if len(attr) > 2:
                text = attr[2]

            if isinstance(info[name], tuple):
                full_text = text + info[name][0]
                color = info[name][1]
            else:
                full_text = text + info[name]

            _print_colored(full_text, color, end='')
            _print_colored(' ' * (spaces[name] - len(info[name])), end='')
        _print_colored()


@output_control
def clear():
    pass


def _print_colored(text='', color='blue', end='\n'):
    global _message
    global _color_policy

    if _color_policy:
        if color:
            _message += colored(text, color)
        else:
            _message += colored(text)
    else:
        _message += text

    if end == '\n':
        logger.info(_message)
        _message = ''



import mistune

def markdown_to_latex(content):
    renderer = LaTeXRenderer()
    markdown = mistune.Markdown(renderer=renderer)
    return markdown(content)

col_formatting = '|'

class LaTeXRenderer(mistune.Renderer):

    def block_quote(self, text):
        return '{}'.format(text)

    def block_html(self, html):
        return ''

    def inline_html(self, text):
        return ''

    def header(self, text, level, raw):
        if level == 1:
            return '\n\\section{{{}}}\n\n'.format(text)
        elif level == 2:
            return '\n\\subsection{{{}}}\n\n'.format(text)
        elif level == 3:
            return '\n\\subsubsection{{{}}}\n\n'.format(text)
        elif level == 4:
            return '\n\\paragraph{{{}}}\n\n'.format(text)
        elif level == 5:
            return '\n\\paragraph{{{}}}\n\n'.format(text)
        elif level == 6:
            return '\n\\subparagraph{{{}}}\n\n'.format(text)

    def hrule(self):
        return ''

    def list(self, body, ordered):
        if ordered:
            return '\\begin{{enumerate}}{}\\end{{enumerate}}\n\n'.format(body)
        else:
            return '\\begin{{itemize}}{}\\end{{itemize}}\n\n'.format(body)

    def list_item(self, text):
        return '\item {}\n'.format(text)

    def paragraph(self, text):
        return '{}\n\n'.format(text)

    def table(self, header, body):
        global col_formatting
        table_latex = \
'''\\def\\arraystretch{1.5}
\\begin{tabular}{%s}
  \\hline
%s
%s
  \\hline
\\end{tabular}
''' % (col_formatting, header, body)
        col_formatting = '|'
        return table_latex

    def table_row(self, content):
        content = content.split('&')[:-1]
        return '  {}\\\\\n  \hline\n'.format('&'.join(content))

    def table_cell(self, content, **flags):
        global col_formatting
        if flags['header']:
            if flags['align'] == 'right':
                col_formatting += ' r |'
            elif flags['align'] == 'center':
                col_formatting += ' c |'
            elif flags['align'] == 'left':
                col_formatting += ' l |'
            else:
                col_formatting += ' l |'
            return '\\textbf{{{}}} & '.format(content)
        else:
            return '{} & '.format(content)

    def link(self, link, title, content):
        return '\\href{{{}}}{{{}}}'.format(link, content)

    def autolink(self, link, is_email=False):
        return '\\href{{{0}}}{{{0}}}'.format(link)

    def block_code(self, code, language):
        result = '\\begin{{verbatim}}\n{}\n\\end{{verbatim}}\n'.format(code)
        return result

    def codespan(self, text):
        return '\\texttt{{{}}}'.format(text)

    def double_emphasis(self, text):
        return '\\textbf{{{}}}'.format(text)

    def emphasis(self, text):
        return '\\emph{{{}}}'.format(text)

    def image(self, src, title, alt_text):
        return \
'''\\begin{{figure}}[!h]
  \\includegraphics{{{}}}
\\end{{figure}}'''.format(src)

    def strikethrough(self, text):
        return '\sout{{{}}}'.format(text)

    def text(self, text):
        return text

    def linebreak(self):
        return ''

    def newline(self):
        return ''



from os.path import join, isfile, isdir
from os import getcwd
from kooki.config import get_kooki_dir, get_kooki_dir_recipes, get_kooki_dir_jars


def search_jar(jar):
    ret_jar_path = None
    user_jars_dir = get_kooki_dir_jars()
    jar_path = join(user_jars_dir, jar)
    if isdir(jar_path):
        ret_jar_path = jar_path
    return ret_jar_path


def search_recipe(recipe):
    ret_recipe_path = None
    user_recipes_dir = get_kooki_dir_recipes()
    recipe_path = join(user_recipes_dir, recipe, 'recipe.py')
    if isfile(recipe_path):
        ret_recipe_path = recipe_path
    return ret_recipe_path


def search_file(jars, recipe, filename):
    ret_file_path = None
    ret_file_path = search_file_in_local(filename)
    if not ret_file_path:
        ret_file_path = search_file_in_jars(jars, recipe, filename)
    return ret_file_path


def search_file_in_local(filename):
    ret_file_path = None
    file_path = join(getcwd(), filename)
    if isfile(file_path):
        ret_file_path = file_path
    return ret_file_path


def search_file_in_jars(jars, recipe, filename):
    user_jars_dir = get_kooki_dir_jars()
    for jar in jars:
        jar_path = join(user_jars_dir, jar)
        ret_file_path = search_file_in_jar(jar_path, recipe, filename)
        if ret_file_path: break
    return ret_file_path


def search_file_in_jar(jar_path, recipe, filename):
    ret_file_path = search_file_in_jar_recipe(jar_path, recipe, filename)
    if not ret_file_path:
        ret_file_path = search_file_in_jar_local(jar_path, filename)
    return ret_file_path


def search_file_in_jar_recipe(jar_path, recipe, filename):
    ret_file_path = None
    recipe_jar_path = join(jar_path, recipe)
    file_path = join(recipe_jar_path, filename)
    if isfile(file_path):
        ret_file_path = file_path
    return ret_file_path


def search_file_in_jar_local(jar_path, filename):
    ret_file_path = None
    file_path = join(jar_path, filename)
    if isfile(file_path):
        ret_file_path = file_path
    return ret_file_path



import os, yaml

from kooki.tools import write_file
from kooki.exception import YamlErrorConfigFileBadType, YamlErrorConfigFileParsing, NoConfigFileFound

resource_dir_env = 'KOOKI_DIR'
resource_dir_default = '~/.kooki'

jar_manager_env = 'KOOKI_JAR_MANAGER'
jar_manager_default = ['https://gitlab.com/kooki/jar_manager/raw/master/jars.yml']

recipe_manager_env = 'KOOKI_RECIPE_MANAGER'
recipe_manager_default = ['https://gitlab.com/kooki/recipe_manager/raw/master/recipes.yml']


def get_kooki_dir():
    resource_dir = os.environ.get(resource_dir_env)
    if not resource_dir:
        resource_dir = os.path.expanduser(resource_dir_default)
    return resource_dir


def get_kooki_dir_jars():
    resources_dir = get_kooki_dir()
    jars_dir = os.path.join(resources_dir, 'jars')
    return jars_dir


def get_kooki_dir_recipes():
    resources_dir = get_kooki_dir()
    recipes_dir = os.path.join(resources_dir, 'recipes')
    return recipes_dir


def get_kooki_jar_manager():
    jar_manager = os.environ.get(jar_manager_env)
    if jar_manager:
        jar_manager = jar_manager.split(';')
    else:
        jar_manager = jar_manager_default
    return jar_manager


def get_kooki_recipe_manager():
    recipe_manager = os.environ.get(recipe_manager_env)
    if recipe_manager:
        recipe_manager = recipe_manager.split(';')
    else:
        recipe_manager = recipe_manager_default
    return recipe_manager


def read_config_file(config_file_name):
    if os.path.isfile(config_file_name):
        with open(config_file_name, 'r') as stream:
            try:
                content = stream.read()
                config = yaml.safe_load(content)
                if not isinstance(config, dict):
                    raise YamlErrorConfigFileBadType(type(config))

                return config
            except yaml.YAMLError as exc:
                raise YamlErrorConfigFileParsing(exc)
    else:
        raise NoConfigFileFound(config_file_name)


def generate_config_file(config_file_name, config):
    kooki_config_file = {}
    kooki_config_file[config['name']] = config
    kooki_config_file_yaml = yaml.safe_dump(kooki_config_file)
    write_file(config_file_name, kooki_config_file_yaml)



import em
from kooki.tools import get_front_matter
from collections import OrderedDict


def apply_template(data, metadata):
    interpreter = em.Interpreter()
    interpreter.setPrefix('@')
    result = None

    if isinstance(data, list):
        result = []
        for content in data:
            result.append(interpreter.expand(content, metadata))

    elif isinstance(data, OrderedDict):
        result = OrderedDict()
        for file_path, file_content in data.items():
            front_matter, content = get_front_matter(file_content)
            metadata.update(front_matter)
            result[file_path] = interpreter.expand(content, metadata)

    elif isinstance(data, dict):
        result = {}
        for file_path, file_content in data.items():
            front_matter, content = get_front_matter(file_content)
            metadata.update(front_matter)
            result[file_path] = interpreter.expand(content, metadata)

    elif isinstance(data, str):
        result = ''
        result = interpreter.expand(data, metadata)

    else:
        raise Exception('templating bad data type {}'.format(type(data)))

    return result



import yaml, pretty_output

from karamel.command import Command

from kooki.version import __version__
from kooki.tools import read_file

__command__ = 'check'
__description__ = 'Check version of kooki freeze'

class CheckCommand(Command):

    def __init__(self):
        super(CheckCommand, self).__init__(__command__, __description__)
        self.add_argument('--no-color', help='The output has no color.', action='store_true')
        self.add_argument('--no-output', help='There is no output.', action='store_true')

    def callback(self, args):
        pretty_output.set_output_policy(not args.no_output)
        pretty_output.set_color_policy(not args.no_color)
        pretty_output.set_debug_policy(args.debug)
        pretty_output.command(self.command)

        freeze_file = read_file('.kooki_freeze')
        freeze_data = yaml.safe_load(freeze_file)
        check_kooki_freeze(__version__, freeze_data)


def check_kooki_freeze(version, freeze_data):

    # check kooki version
    Output.start_step('kooki')
    if freeze_data['kooki'] != version:
        Output._print_colored('version needed ', 'blue', end='')
        Output._print_colored(freeze_data['kooki'], 'cyan', end='')
        Output._print_colored(', version installed ', 'blue', end='')
        Output._print_colored(version, 'cyan', end='')
        Output._print_colored(' [bad kooki version]', 'red')
    else:
        Output._print_colored('version ', 'blue', end='')
        Output._print_colored(version, 'cyan', end='')
        Output._print_colored(' [good kooki version]', 'green')

    jars = get_jars()

    Output.start_step('jars')
    for jar in freeze_data['repositories']:
        version_freezed = freeze_data['repositories'][jar]['version']
        Output._print_colored(jar, 'blue')
        Output._print_colored('  freezed: ', 'yellow', end='')
        Output._print_colored(version_freezed, 'cyan')
        Output._print_colored('  installed: ', 'yellow', end='')
        if jar in jars:
            version_installed = jars[jar]['version']
            if version_freezed == version_installed:
                Output._print_colored(version_installed, 'green')
            else:
                Output._print_colored(version_installed, 'red', end='')
                Output._print_colored(' [bad version installed]', 'red')
        else:
            Output._print_colored('[jar not installed]', 'red')



from codecs import open
import yaml
import os


def get_extension(file_name):
    tmp = file_name.split('.')
    if len(tmp) > 1:
        return tmp[-1]
    else:
        return ''


def get_front_matter(content):
    first_line = content.split('\n', 1)[0]
    front_matter = {}

    if first_line == '---':
        content_splitted = content.split('---\n')
        if len(content_splitted) >= 3:
            metadata_content = content_splitted[1]
            content = '---\n'.join(content_splitted[2:])
            front_matter = yaml.load(metadata_content)
        else:
            content = '---\n'.join(content_splitted)

    return front_matter, content


def read_file(file_name):
    try:
        stream = open(file_name, 'r', encoding='utf8')
        content = stream.read()
        stream.close()
        return content
    except IOError as e:
        raise RuntimeError('No such file: \'{0}\''.format(file_name))


def write_file(file_name, content):
    path = '/'.join(file_name.split('/')[:-1])
    if path != '' and not os.path.isdir(path):
        os.makedirs(path)

    stream = open(file_name, 'w', encoding='utf8')
    stream.write(content)
    stream.close()
    return content



import mistune

def markdown_to_html(content):
    renderer = HTMLRenderer()
    markdown = mistune.Markdown(renderer=renderer)
    return markdown(content)


class HTMLRenderer(mistune.Renderer):

    def block_code(self, code, language):
        result = '<pre><code>{}</code></pre>'.format(code)
        return result

    def block_quote(self, text):
        return '<blockquote>{}</blockquote>'.format(text)

    def block_html(self, html):
        return html

    def header(self, text, level, raw):
        return '<h{0}>{1}</h{0}>'.format(level, text)

    def hrule(self):
        return '<hr/>'

    def list(self, body, ordered):
        if ordered:
            return '<ol>{}</ol>'.format(body)
        else:
            return '<ul>{}</ul>'.format(body)

    def list_item(self, text):
        return '<li>{}</li>'.format(text)

    def paragraph(self, text):
        return '<p>{}</p>'.format(text)

    def autolink(self, link, is_email=False):
        return '<a href="{0}">{0}</a>'.format(link)

    def codespan(self, text):
        return '<code>{}</code>'.format(text)

    def double_emphasis(self, text):
        return '<strong>{}</strong>'.format(text)

    def emphasis(self, text):
        return '<em>{}</em>'.format(text)

    def image(self, src, title, alt_text):
        return '<img src="{}" title="{}" alt="{}"/>'.format(src, title, alt_text)

    def link(self, link, title, content):
        return '<a href="{}" title="{}">{}</a>'.format(link, title, content)

    def strikethrough(self, text):
        return '<strike>{}</strike>'.format(text)

    def text(self, text):
        return text

    def inline_html(self, text):
        return text

    def linebreak(self):
        return ''

    def newline(self):
        return ''



import subprocess, os, shutil, tempfile

from kooki.tools import write_file
import pretty_output


def latex_to_pdf(name, content):

    pretty_output.start_step('pdf')
    infos = []

    temp_dir = tempfile.mkdtemp()

    tex_file_name = '{0}.tex'.format(name)
    pdf_file_name = '{0}.pdf'.format(name.split('/')[-1])
    relative_output_path = '/'.join(name.split('/')[:-1])
    absolute_output_path = os.path.join(os.getcwd(), relative_output_path)

    tex_file_path = os.path.join(temp_dir, tex_file_name)
    pdf_file_path = os.path.join(temp_dir, pdf_file_name)

    write_file(tex_file_path, content)

    command = 'xelatex -interaction=nonstopmode -halt-on-error -output-directory={1} {0}'.format(tex_file_name, temp_dir)
    log_file = os.path.join(temp_dir, 'xelatex.log')

    with open(log_file, "w") as f:
        subprocess.call(command, shell=True, stdout=f)
        subprocess.call(command, shell=True, stdout=f)

    if pretty_output._debug_policy:
        pretty_output.info('XeLaTeX output: cat {0}'.format(log_file))

    if os.path.isfile(pdf_file_path):
        shutil.copy(pdf_file_path, absolute_output_path)
        infos.append({'name': pdf_file_name, 'status': '[created]', 'info': ''})
    else:
        infos.append({'name': pdf_file_name, 'status': ('[missing]', 'red'), 'info': ''})

    pretty_output.infos(infos, [('name', 'blue'), ('status', 'green'), ('info', 'cyan')])



import argparse, em, yaml, os, pretty_output

from karamel.command import Command

from kooki.tools import write_file
from kooki.config import get_kooki_dir


__command__ = 'new'
__description__ = 'Create a new kooki.yaml file'

default_name = 'index'
default_recipe = 'web'
default_jars = ['default']
default_template = 'default.html'

default_content = '''# @document.name

This is a default document generated by *kooki new*.

Enjoy using **kooki**.

*@document.author*
'''

default_metadata = '''document:
    name: Kooki Document
    author: The developpers
'''

class NewCommand(Command):

    def __init__(self):
        super(NewCommand, self).__init__(__command__, __description__)
        self.add_argument('--config-file', default='kooki.yaml')
        self.add_argument('-n', '--name', help='document name')
        self.add_argument('-t', '--template', help='document template')
        self.add_argument('-r', '--recipe', help='document recipe')
        self.add_argument('-j', '--jars', nargs='+', help='documents kooki jars')
        self.add_argument('-m', '--metadata', nargs='+', help='documents metadata sources')
        self.add_argument('-c', '--content', nargs='+', help='documents content sources')
        self.add_argument('--no-color', help='The output has no color.', action='store_true')
        self.add_argument('--no-output', help='There is no output.', action='store_true')

    def callback(self, args):
        pretty_output.set_output_policy(not args.no_output)
        pretty_output.set_color_policy(not args.no_color)
        pretty_output.set_debug_policy(args.debug)
        pretty_output.command(self.command)

        pretty_output.start_step(args.config_file)
        self.check_override_config_file(args.config_file)

        name = self.handle_name(args)
        recipe = self.handle_recipe(args)
        jars = self.handle_jars(args)
        template = self.handle_template(args)
        metadata = self.handle_metadata(args)
        content = self.handle_content(args)

        config = {
            'name': name,
            'template': template,
            'recipe': recipe,
            'jars': jars,
            'metadata': metadata,
            'content': content}

        generate_config_file(args.config_file, config)

    def check_override_config_file(self, config_file):
        if os.path.exists(config_file):
            response_ok = False
            while not response_ok:
                print('kooki.yaml already exist.')
                response = input("overide it [y/n] (y): ")
                if response == 'n':
                    raise RuntimeError('Existing kooki.yaml has not be changed.')
                elif response == 'y':
                    response_ok = True

    def handle_name(self, args, document_info):
        pretty_output.start_step('name')
        if args.name:
            document_info['name'] = args.name
        else:
            document_info['name'] = default_name
        pretty_output.info(document_info['name'])

    def handle_recipe(self, args, document_info):
        pretty_output.start_step('recipe')
        if args.recipe:
            document_info['recipe'] = args.recipe
        else:
            document_info['recipe'] = default_recipe
        pretty_output.info(document_info['recipe'])

    def handle_jars(self, args, document_info):
        pretty_output.start_step('jars')
        if args.jars:
            document_info['jars'] = []
            for jar in args.jars:
                document_info['jars'].append(jar)
        else:
            document_info['jars'] = default_jars
        pretty_output.info(yaml.safe_dump(document_info['jars'])[:-1])

    def handle_template(self, args, document_info):
        pretty_output.start_step('template')
        if args.template:
            document_info['template'] = args.template
        else:
            document_info['template'] = default_template
        pretty_output.info(document_info['template'])

    def handle_metadata(self, args, document_info):
        pretty_output.start_step('metadata')
        if args.metadata:
            document_info['metadata'] = []
            for metadata in args.metadata:
                document_info['metadata'].append(metadata)
        else:
            document_info['metadata'] = ['metadata.yaml']
            write_file('metadata.yaml', default_metadata)
        pretty_output.info(yaml.safe_dump(document_info['metadata'])[:-1])

    def handle_content(self, args, document_info):
        pretty_output.start_step('content')
        if args.content:
            document_info['content'] = []
            for content in args.content:
                document_info['content'].append(content)
        else:
            document_info['content'] = ['content.md']
            write_file('content.md', default_content)
        pretty_output.info(yaml.safe_dump(document_info['content'])[:-1])



#!/usr/bin/env python
import os, atexit
from setuptools import setup, find_packages
from setuptools.command.install import install
from subprocess import check_call

__version__ = '0.7.1'

git_recipes = [
    'https://gitlab.com/kooki/recipe/md.git',
    'https://gitlab.com/kooki/recipe/tex.git',
    'https://gitlab.com/kooki/recipe/web.git']

git_jars = [
    'https://gitlab.com/kooki/jar/default.git']

resource_dir_env = 'KOOKI_DIR'
resource_dir_default = '~/.kooki'


def get_kooki_dir():
    resource_dir = os.environ.get(resource_dir_env)
    if not resource_dir:
        resource_dir = os.path.expanduser(resource_dir_default)
    return resource_dir


def get_kooki_dir_jars():
    resources_dir = get_kooki_dir()
    jars_dir = os.path.join(resources_dir, 'jars')
    return jars_dir


def get_kooki_dir_recipes():
    resources_dir = get_kooki_dir()
    recipes_dir = os.path.join(resources_dir, 'recipes')
    return recipes_dir


def clone_repositories(path, repositories):
    tmp = os.getcwd()
    os.makedirs(path)
    os.chdir(path)
    for repository in repositories:
        command = 'git clone {}'.format(repository)
        check_call(command.split())
    os.chdir(tmp)


def post_install():
    kooki_dir_recipes = get_kooki_dir_recipes()
    clone_repositories(kooki_dir_recipes, git_recipes)
    kooki_dir_jars = get_kooki_dir_jars()
    clone_repositories(kooki_dir_jars, git_jars)


class NewInstall(install):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        atexit.register(post_install)


setup(
    name='kooki',
    version=__version__,
    description='The ultimate document generator.',
    author='Noel Martignoni',
    include_package_data=True,
    package_data={'kooki.config': ['format.yaml']},
    author_email='noel@martignoni.fr',
    url='https://gitlab.com/kooki/kooki',
    scripts=['scripts/kooki', 'scripts/kooki-recipe', 'scripts/kooki-jar'],
    install_requires=['empy', 'pyyaml', 'toml', 'requests', 'termcolor', 'mistune', 'karamel', 'munch'],
    packages=find_packages(exclude=['tests*']),
    test_suite='tests',
    cmdclass={
        'install': NewInstall,
    },
)



from kooki.tools import get_extension
import yaml
import toml
import json
import munch


class Metadata(munch.Munch):

    def update(self, *args, **kwargs):
        for arg in args:
            if isinstance(arg, dict):
                super().update(arg)
            else:
                raise Exception('arg provided is not a dict')
        super().update(**kwargs)


def parse_metadata(document_metadata):
    metadata = Metadata()
    for file_path, file_content in document_metadata.items():
        new_metadata = load(file_path, file_content)
        metadata = data_merge(new_metadata, metadata)
    return Metadata.fromDict(metadata)


def load(file_path, file_content):

    extension = get_extension(file_path)

    if extension == 'yaml' or extension == 'yml':
        metadata = yaml.safe_load(file_content)
    elif extension == 'toml':
        metadata = toml.loads(file_content)
    elif extension == 'json':
        metadata = json.loads(file_content)
    else:
        raise Exception('No loader for this type of file \'.{}\''.format(extension))

    return metadata


def data_merge(a, b):

    class MergeError(Exception):
        pass

    key = None

    try:
        if a is None or isinstance(a, str) or isinstance(a, bytes) or isinstance(a, int) or isinstance(a, float):
            a = b
        elif isinstance(a, list):
            if isinstance(b, list):
                a.extend(b)
            else:
                a.append(b)
        elif isinstance(a, dict):
            if isinstance(b, dict):
                for key in b:
                    if key in a:
                        a[key] = data_merge(a[key], b[key])
                    else:
                        a[key] = b[key]
            else:
                raise MergeError('Cannot merge non-dict "%s" into dict "%s"' % (b, a))
        else:
            raise MergeError('NOT IMPLEMENTED "%s" into "%s"' % (b, a))
    except TypeError as e:
        raise MergeError('TypeError "%s" in key "%s" when merging "%s" into "%s"' % (e, key, b, a))
    return a



class KookiException(Exception):
    pass

no_config_file_found_message = \
'''
No '{}' file found.
Create one by executing 'kooki new'.
'''

class NoConfigFileFound(KookiException):
    def __init__(self, config_file_name):
        message = no_config_file_found_message.format(config_file_name)
        super(NoConfigFileFound, self).__init__(message)


yaml_error_config_file_parsing_message = \
'''Yaml error during config file parsing.
Please check the format of your config file.'''

class YamlErrorConfigFileParsing(KookiException):
    def __init__(self, exc):
        message = yaml_error_config_file_parsing_message
        super(YamlErrorConfigFileParsing, self).__init__(message)


yaml_error_config_file_bad_type_message = \
'''The Yaml parsed should be a dict and it is a {}.
Please check the format of your config file.'''

class YamlErrorConfigFileBadType(KookiException):
    def __init__(self, type_found):
        message = yaml_error_config_file_bad_type_message.format(type_found)
        super(YamlErrorConfigFileBadType, self).__init__(message)
